##  Copyright (C) 2014  Daniel Kraft <d@domob.eu>
##  GNU Octave level-set package.
##
##  This program is free software: you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation, either version 3 of the License, or
##  (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program.  If not, see <http://www.gnu.org/licenses/>.

## -*- texinfo -*-
## @deftypefn {Function File} ls_sign_colourmap ()
## @deftypefnx  {Function File} {@var{cmap} =} ls_sign_colourmap (@var{ax}, @var{n} = 1024)
## 
## Construct a colour map that has a visible ``discontinuity'' between
## positive and negative values.  The range above zero will be red--yellow,
## and the range below will be blue--cyan.
## This colour map is especially suited to show the distinction between
## above-zero and below-zero of a level-set function or for speed fields
## between moving outward and inward.
##
## The colour axis range for which it is used should be given in @var{ax}.
## If @var{n} is given, construct a map consisting of that many entries.
##
## The first form without arguments and outputs uses @code{caxis ()} of the
## current figure and sets the figure's colour map to the result, instead of
## returning the colour map.
##
## Use @code{demo ls_sign_colourmap} to get a quick example.
##
## @seealso{colormap, colorbar}
## @end deftypefn

function cmap = ls_sign_colourmap (ax, n = 1024)
  cmap = NA (n, 3);

  setFig = false;
  if (nargin () == 0)
    if (nargout () > 0)
      usage ("ls_sign_colourmap without arguments only works without outputs");
    endif
    setFig = true;
    ax = caxis ();
  endif

  if (ax(1) >= 0)
    nBelow = 0;
    nAbove = n;
  elseif (ax(2) <= 0)
    nBelow = n;
    nAbove = 0;
  else
    assert (ax(1) < 0 && ax(2) > 0);
    nBelow = round (n * -ax(1) / (ax(2) - ax(1)));
    nAbove = n - nBelow;
    assert (nAbove, round (n * ax(2) / (ax(2) - ax(1))), 1);
  endif

  yellow = [1, 1, 0];
  red = [1, 0, 0];
  blue = [0, 0, 1];
  cyan = [0, 1, 1];

  for i = 1 : nBelow
    theta = (i - 1) / (nBelow - 1);
    cmap(i, :) = theta * cyan + (1 - theta) * blue;
  endfor

  for i = 1 : nAbove
    theta = (i - 1) / (nAbove - 1);
    cmap(i + nBelow, :) = theta * red + (1 - theta) * yellow;
  endfor

  if (setFig)
    colormap (cmap);
  endif
endfunction

% Test the usage check.
%!error <ls_sign_colourmap without arguments>
%!  cmap = ls_sign_colourmap ();

% Demo function that shows how this looks like.
%!demo
%!  n = 100;
%!  x = linspace (-2, 2, n);
%!  y = linspace (-1, 3, n);
%!  [XX, YY] = meshgrid (x, y);
%!  phi = XX.^2 + (YY - 1).^2 - 1;
%!
%!  figure ();
%!  imagesc (x, y, phi);
%!  set (gca (), "ydir", "normal");
%!  ls_sign_colourmap ();
%!  colorbar ();
